@@ -18,6 +18,9 @@
 """nio api module.
 
 This module contains primitives to build Matrix API http requests.
+
+In general these functions are not directly called. One should use an existing
+client like AsyncClient or HttpClient.
 """
 
 
@@ -45,6 +48,8 @@ except ImportError:
 MATRIX_API_PATH = "/_matrix/client/r0"  # type: str
 MATRIX_MEDIA_API_PATH = "/_matrix/media/r0"  # type: str
 
+_FilterT = Union[None, str, Dict[Any, Any]]
+
 
 @unique
 class MessageDirection(Enum):
@@ -96,7 +101,7 @@ class RoomPreset(Enum):
     public_chat = "public_chat"
 
 
-class Api(object):
+class Api:
     """Matrix API class.
 
     Static methods reflecting the Matrix REST API.
@@ -240,13 +245,48 @@ class Api(object):
 
         return "GET", path
 
+    @staticmethod
+    def register(
+            user,           # type: str
+            password=None,  # type: str
+            device_name="",  # type: Optional[str]
+            device_id="",    # type: Optional[str]
+    ):
+        """Register a new user.
+
+        Args:
+            user (str): The fully qualified user ID or just local part of the
+                user ID, to log in.
+            password (str): The user's password.
+            device_name (str): A display name to assign to a newly-created
+                device. Ignored if device_id corresponds to a known device
+            device_id (str): ID of the client device. If this does not
+                correspond to a known client device, a new device will be
+                created.
+        """
+        path = Api._build_path("register")
+
+        content_dict = {
+            "auth": {"type": "m.login.dummy"},
+            "username": user,
+            "password": password,
+        }
+
+        if device_id:
+            content_dict["device_id"] = device_id
+
+        if device_name:
+            content_dict["initial_device_display_name"] = device_name
+
+        return "POST", path, Api.to_json(content_dict)
+
     @staticmethod
     def login(
         user,            # type: str
         password=None,   # type: str
         device_name="",  # type: Optional[str]
         device_id="",    # type: Optional[str]
-        token=None,      # type: str
+        token=None,       # type: str
     ):
         # type: (...) -> Tuple[str, str, str]
         """Authenticate the user.
@@ -352,11 +392,11 @@ class Api(object):
 
     @staticmethod
     def sync(
-        access_token,     # type: str
-        since=None,       # type: Optional[str]
-        timeout=None,     # type: Optional[int]
-        filter=None,      # type: Optional[Dict[Any, Any]]
-        full_state=None   # type: Optional[bool]
+        access_token: str,
+        since:        Optional[str]  = None,
+        timeout:      Optional[int]  = None,
+        filter:       _FilterT       = None,
+        full_state:   Optional[bool] = None,
     ):
         # type: (...) -> Tuple[str, str]
         """Synchronise the client's state with the latest state on the server.
@@ -367,10 +407,15 @@ class Api(object):
             access_token (str): The access token to be used with the request.
             since (str): The room id of the room where the event will be sent
                 to.
-            timeout(int): The maximum time to wait, in milliseconds, before
+            timeout (int): The maximum time to wait, in milliseconds, before
                 returning this request.
-            filter (Dict): A dictionary containing a filter configuration for
-                the request.
+            sync_filter (Union[None, str, Dict[Any, Any]):
+                A filter ID or dict that should be used for this sync request.
+            full_state (bool, optional): Controls whether to include the full
+                state for all rooms the user is a member of. If this is set to
+                true, then all state events will be returned, even if since is
+                non-empty. The timeline will still be limited by the since
+                parameter.
         """
         query_parameters = {"access_token": access_token}
 
@@ -383,9 +428,11 @@ class Api(object):
         if timeout is not None:
             query_parameters["timeout"] = str(timeout)
 
-        if filter is not None:
+        if isinstance(filter, dict):
             filter_json = json.dumps(filter, separators=(",", ":"))
             query_parameters["filter"] = filter_json
+        elif isinstance(filter, str):
+            query_parameters["filter"] = filter
 
         return "GET", Api._build_path("sync", query_parameters)
 
@@ -546,7 +593,7 @@ class Api(object):
     @staticmethod
     def room_kick(access_token, room_id, user_id, reason=None):
         # type (str, str, str, Optional[str]) -> Tuple[str, str, str]
-        """Kick a user from a room.
+        """Kick a user from a room, or withdraw their invitation.
 
         Returns the HTTP method, HTTP path and data for the request.
 
@@ -555,6 +602,7 @@ class Api(object):
             room_id (str): The room id of the room that the user will be
                 kicked from.
             user_id (str): The user_id of the user that should be kicked.
+            reason (str, optional): A reason for which the user is kicked.
         """
         query_parameters = {"access_token": access_token}
 
@@ -571,6 +619,63 @@ class Api(object):
             Api.to_json(body)
         )
 
+    @staticmethod
+    def room_ban(
+        access_token: str,
+        room_id: str,
+        user_id: str,
+        reason: Optional[str] = None,
+    ) -> Tuple[str, str, str]:
+        """Ban a user from a room.
+
+        Returns the HTTP method, HTTP path and data for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            room_id (str): The room id of the room that the user will be
+                banned from.
+            user_id (str): The user_id of the user that should be banned.
+            reason (str, optional): A reason for which the user is banned.
+        """
+
+        path = f"rooms/{room_id}/ban"
+        query_parameters = {"access_token": access_token}
+        body = {"user_id": user_id}
+
+        if reason:
+            body["reason"] = reason
+
+        return (
+            "POST",
+            Api._build_path(path, query_parameters),
+            Api.to_json(body),
+        )
+
+    @staticmethod
+    def room_unban(
+        access_token: str, room_id: str, user_id: str,
+    ) -> Tuple[str, str, str]:
+        """Unban a user from a room.
+
+        Returns the HTTP method, HTTP path and data for the request.
+
+        Args:
+            access_token (str): The access token to be used with the request.
+            room_id (str): The room id of the room that the user will be
+                unbanned from.
+            user_id (str): The user_id of the user that should be unbanned.
+        """
+
+        path = f"rooms/{room_id}/unban"
+        query_parameters = {"access_token": access_token}
+        body = {"user_id": user_id}
+
+        return (
+            "POST",
+            Api._build_path(path, query_parameters),
+            Api.to_json(body),
+        )
+
     @staticmethod
     def room_invite(access_token, room_id, user_id):
         # type (str, str, str) -> Tuple[str, str, str]
@@ -769,12 +874,13 @@ class Api(object):
 
     @staticmethod
     def room_messages(
-        access_token,                     # type: str
-        room_id,                          # type: str
-        start,                            # type: str
-        end=None,                         # type: Optional[str]
-        direction=MessageDirection.back,  # type: MessageDirection
-        limit=10,                         # type: int
+        access_token:   str,
+        room_id:        str,
+        start:          str,
+        end:            Optional[str]    = None,
+        direction:      MessageDirection = MessageDirection.back,
+        limit:          int              = 10,
+        message_filter: _FilterT         = None,
     ):
         # type (...) -> Tuple[str, str]
         """Get room messages.
@@ -789,11 +895,15 @@ class Api(object):
             end (str): The token to stop returning events at.
             direction (MessageDirection): The direction to return events from.
             limit (int): The maximum number of events to return.
+            message_filter (Union[None, str, Dict[Any, Any]]):
+                A filter ID or dict that should be used for this room messages
+                request.
+
         """
         query_parameters = {
             "access_token": access_token,
             "from": start,
-            "limit": limit
+            "limit": limit,
         }
 
         if end:
@@ -812,6 +922,12 @@ class Api(object):
         else:
             query_parameters["dir"] = "b"
 
+        if isinstance(message_filter, dict):
+            filter_json = json.dumps(message_filter, separators=(",", ":"))
+            query_parameters["filter"] = filter_json
+        elif isinstance(message_filter, str):
+            query_parameters["filter"] = message_filter
+
         path = "rooms/{room}/messages".format(room=room_id)
 
         return "GET", Api._build_path(path, query_parameters)
@@ -962,7 +1078,7 @@ class Api(object):
                 updated for the device.
         """
         query_parameters = {"access_token": access_token}
-        path = "devices/{}".format(device_id)
+        path = "devices/{}".format(quote(device_id))
 
         return (
             "PUT",
